@@ -79,7 +79,7 @@ void meta_window_get_input_rect (const MetaWindow *window, MetaRectangle *rect);
 void meta_window_get_outer_rect (const MetaWindow *window, MetaRectangle *rect);
 MetaScreen *meta_window_get_screen (MetaWindow *window);
 MetaDisplay *meta_window_get_display (MetaWindow *window);
-Window meta_window_get_xwindow (MetaWindow *window);
+unsigned long meta_window_get_xwindow (MetaWindow *window);
 MetaWindowType meta_window_get_window_type (MetaWindow *window);
 Atom meta_window_get_window_type_atom (MetaWindow *window);
 MetaWorkspace *meta_window_get_workspace (MetaWindow *window);
@@ -141,14 +141,20 @@ void     meta_window_foreach_ancestor         (MetaWindow            *window,
 
 MetaMaximizeFlags meta_window_get_maximized (MetaWindow *window);
 gboolean          meta_window_is_fullscreen (MetaWindow *window);
+gboolean          meta_window_is_monitor_sized (MetaWindow *window);
 gboolean          meta_window_is_on_primary_monitor (MetaWindow *window);
+gint             *meta_window_get_all_monitors (MetaWindow *window, gsize *length);
 
 gboolean meta_window_is_demanding_attention (MetaWindow *window);
 gboolean meta_window_is_urgent (MetaWindow *window);
-gboolean meta_window_is_mapped (MetaWindow  *window);
-gboolean meta_window_toplevel_is_mapped (MetaWindow  *window);
+
+gboolean          meta_window_requested_bypass_compositor (MetaWindow *window);
+gboolean          meta_window_requested_dont_bypass_compositor (MetaWindow *window);
+
 gboolean meta_window_get_icon_geometry (MetaWindow    *window,
                                         MetaRectangle *rect);
+void meta_window_set_icon_geometry (MetaWindow    *window,
+                                    MetaRectangle *rect);
 void meta_window_maximize   (MetaWindow        *window,
                              MetaMaximizeFlags  directions);
 void meta_window_unmaximize (MetaWindow        *window,
@@ -157,6 +163,7 @@ void        meta_window_minimize           (MetaWindow  *window);
 void        meta_window_unminimize         (MetaWindow  *window);
 void        meta_window_raise              (MetaWindow  *window);
 void        meta_window_lower              (MetaWindow  *window);
+void        meta_window_reset_opacity      (MetaWindow *window);
 const char *meta_window_get_title (MetaWindow *window);
 MetaWindow *meta_window_get_transient_for (MetaWindow *window);
 Window      meta_window_get_transient_for_as_xid (MetaWindow *window);
@@ -177,4 +184,7 @@ cairo_region_t *meta_window_get_frame_bounds (MetaWindow *window);
 
 MetaWindow *meta_window_get_tile_match (MetaWindow *window);
 
+gboolean meta_window_can_tile (MetaWindow *window, MetaTileMode mode);
+gboolean meta_window_tile (MetaWindow *window, MetaTileMode mode, gboolean snap);
+
 #endif
